from loguru import logger
import json
from pathlib import Path


def load_dataherb_config(config_path=None):
    """Loads the dataherb config file.

    Load the content from the specified file as the config. The config file has to be json.
    """

    if config_path is None:
        home = Path.home()
        config_path = home / ".dataherb/config.json"

    logger.debug(f"Using {config_path} as config file for dataherb")
    with open(config_path, "r") as f:
        conf = json.load(f)

    if not conf.get("workdir"):
        logger.error(
            f"Please specify working directory in the config file using the key workdir"
        )
    elif conf.get("workdir", "").startswith("~"):
        home = Path.home()
        conf["workdir"] = str(home / conf["workdir"][2:])

    return conf



import json
from pathlib import Path

from loguru import logger

from dataherb.core.base import Herb
from dataherb.core.search import search_by_ids_in_flora as _search_by_ids_in_flora
from dataherb.core.search import (
    search_by_keywords_in_flora as _search_by_keywords_in_flora,
)
from dataherb.fetch.remote import get_data_from_url as _get_data_from_url
from dataherb.parse.model_json import MetaData


class Flora(object):
    """
    DataHerb is the container of datasets.
    """

    def __init__(self, flora):
        """
        :param flora: API of the DataHerb service, defaults to dataherb official or list of Herbs, defaults to everything from the API
        """

        if isinstance(flora, str):
            self.workdir = Path(flora).parent.parent
            self.flora_config = flora
            flora = self._get_flora(flora)
        else:
            raise Exception(f"flora must be a json file or a url. ({flora})")

        logger.debug(f"flora workdir {self.workdir}")

        self.flora = flora

    def _get_flora(self, flora_config):
        """
        _get_flora fetch flora from the provided API.
        """
        if Path(flora_config).exists():
            with open(flora_config, "r") as f:
                flora = json.load(f)
        else:
            # assuming the config is a url if the local file does not exist
            flora_request = _get_data_from_url(flora_config)

            if not flora_request.status_code == 200:
                raise Exception(
                    "Could not download dataherb flora from remote. status code: {}".format(
                        flora_request.status_code
                    )
                )
            else:
                flora = flora_request.json()

        # Convert herbs to objects
        flora = [
            Herb(herb, base_path=self.workdir / f'{herb.get("id", "")}')
            for herb in flora
        ]

        return flora

    def add(self, herb):
        """
        add add a herb to the flora
        """
        if isinstance(herb, MetaData):
            herb = Herb(herb.metadata)
        elif isinstance(herb, dict):
            herb = Herb(herb)
        elif isinstance(herb, Herb):
            pass
        else:
            raise Exception(f"Input herb type ({type(herb)}) is not supported.")

        logger.debug(f"metadata: {herb.metadata}")

        for id in [i.id for i in self.flora]:
            if id == herb.id:
                raise Exception(f"herb id = {herb.id} already exists")

        self.flora.append(herb)
        self.save()

    def save(self, path=None):
        """save flora metadata to json file"""

        if path is None:
            path = self.flora_config

        logger.debug(
            f"type of a herb in flora: {type(self.flora[0])}\n{self.flora[0].metadata}"
        )

        serialized_flora = []
        for h in self.flora:
            logger.debug(f"herb (type {type(h)}): {h}")
            serialized_flora.append(h.metadata)

        with open(path, "w") as fp:
            json.dump(
                serialized_flora, fp, sort_keys=True, indent=4, separators=(",", ": ")
            )

    def search(self, keywords):
        """
        search finds the datasets that matches the keywords

        :param keywords: [description]
        :type keywords: [type]
        :return: [description]
        :rtype: [type]
        """
        if isinstance(keywords, str):
            keywords = [keywords]

        return _search_by_keywords_in_flora(self.flora, keywords)

    def herb_meta(self, id):
        """
        herb loads the dataset

        :param id: herb id of the dataset
        :type id: str
        """

        herbs = _search_by_ids_in_flora(self.flora, id)

        if herbs:
            herb = herbs[0]

            herb = herb.get("herb")

            return herb.metadata
        else:
            return

    def herb(self, id):
        """
        herb loads the dataset as dataframes.

        :param id: herb id
        :type id: str
        """

        herbs = _search_by_ids_in_flora(self.flora, id)
        if len(herbs) > 1:
            logger.error(
                f"Found multiple datasets with id {id}, please fix this in your flora data json file, e.g, WORKDIRECTORY/flora/flora.json."
            )

        if herbs:
            herb = herbs[0]
            herb = herb.get("herb")
            return herb
        else:
            logger.error(f"Could not find herb {id}")
            return


if __name__ == "__main__":

    from datapackage import Resource

    fl = Flora(flora="/Users/leima/dataherb/flora/flora.json")

    hb = fl.herb("git-data-science-job")

    print(f"herb base_path: {hb.base_path}")

    rs = hb.resources[0]

    rs_1 = Resource(rs.descriptor, base_path=str(hb.base_path))

    print(f"{rs.tabular}")

    # rs_2.read()

    rs.read()

    print(hb.get_resource(path="dataset/stackoverflow_job_listing.csv"))

    logger.debug("End of Game")



def flatten_dict(nested_dict, sep=None):
    """
    flatten_dict flattens a dictionary,

    The flattened keys are joined using a separater which is default to '__'.

    :param nested_dict: input nested dictionary to be flattened.
    :type nested_dict: dict
    :param sep: seperator for the joined keys, defaults to __
    :type sep: str, optional
    :return: flattened dictionar
    :rtype: dict
    """
    if sep is None:
        sep = "__"
    res = {}

    def flatten(x, name=""):
        if type(x) is dict:
            for a in x:
                flatten(x[a], name + a + sep)
        elif type(x) is list:
            i = 0
            for a in x:
                flatten(a, name + str(i) + sep)
                i += 1
        else:
            res[name[:-2]] = x

    flatten(nested_dict)

    return res



from abc import abstractclassmethod
from loguru import logger
from dataherb.flora import Flora


class SourceModel:
    """
    Model is the base class for
    """

    def __init__(self, path_to_datasets: str) -> None:
        self.path_to_datasets = path_to_datasets
        self.metas = []

    @abstractclassmethod
    def fetch_metadata(self):
        raise NotImplementedError(f"Please implement this method")


class SaveModel:
    """
    SaveModel take the models and save them as files to serve as the database.
    """

    def __init__(self, flora, workdir, **kargs) -> None:

        if isinstance(flora, str):
            flora = Flora(flora)

        self.flora = flora
        self.workdir = workdir
        self.kargs = kargs

    @abstractclassmethod
    def save_json(self) -> None:
        raise NotImplementedError("Please implement save_json method")

    @abstractclassmethod
    def save_markdown(self) -> None:
        raise NotImplementedError("Please implement save_markdown method")

    @abstractclassmethod
    def save_all(self) -> None:
        raise NotImplementedError("Please implement save_all method")



import csv
import os
from collections import OrderedDict
from pathlib import Path

import ruamel.yaml
from loguru import logger
from ruamel.yaml.representer import RoundTripRepresenter

IGNORED_FOLDERS_AND_FILES = [".git", ".dataherb", ".vscode"]

# Add representer to ruamel.yaml for OrderedDict
class MyRepresenter(RoundTripRepresenter):
    pass


ruamel.yaml.add_representer(
    OrderedDict, MyRepresenter.represent_dict, representer=MyRepresenter
)
yaml = ruamel.yaml.YAML()
yaml.Representer = MyRepresenter

MESSAGE_CODE = {
    "MISSING": lambda x: f"{x} is missing",
    "FILE_NOT_FOUND": lambda x: f"{x} was not found",
    "FILE_FOUND": lambda x: f"{x} was found",
    "EXISTS": lambda x: f"{x} exists",
    "FREE_MESSAGE": lambda x: f"{x}",
}

STATUS_CODE = {
    "UNKNOWN": "unknown",
    "SUCCESS": "success",
    "WARNING": "warning",
    "ERROR": "error",
}


class MetaData(object):
    def __init__(self):
        self.dataherb_folder = ".dataherb"
        self.metadata_file = "metadata.yml"
        self.template = OrderedDict(
            {
                "name": "",
                "description": "",
                "contributors": [{"name": "", "github": ""}],
                "data": [],
                "references": [{"name": "", "link": ""}],
            }
        )

    def parse_structure(self, folder=None):

        if folder is None:
            folder = "."

        tree_f = []
        tree_d = []
        for root, dirs, files in os.walk(folder):
            for d in dirs:
                if d not in IGNORED_FOLDERS_AND_FILES:
                    tree_d.append(os.path.relpath(os.path.join(root, d), folder))
            for f in files:
                tree_f.append(os.path.relpath(os.path.join(root, f), folder))

        self.tree = tree_f

        return self.tree

    def parse_csv(self, csv_file):
        """
        parse_csv parses the csv files for metadata generation
        """

        with open(csv_file, "r") as f:
            reader = csv.reader(f)
            columns = next(reader)

        fields = []
        for col in columns:
            fields.append({"name": col, "description": ""})

        return fields

    def _generate_leaf(self, path, meta_input):

        name = meta_input.get("name", "")
        description = meta_input.get("description", "")
        updated_at = meta_input.get("updated_at", "")

        file_format = path.split(".")[-1]
        if len(file_format) >= 10:
            logger.error(f"The format of file {path} could not be determined!")
            file_format = ""

        file_size = os.stat(path).st_size

        if file_format == "csv":
            fields = self.parse_csv(path)
        else:
            fields = [{"name": "", "description": ""}, {"name": "", "description": ""}]

        res = {
            "name": name,
            "description": description,
            "path": path,
            "format": file_format,
            "size": file_size,
            "updated_at": updated_at,
            "fields": fields,
        }

        return res

    def append_leaf(self, dataset_file, meta_input):

        existing_leaves = self.template["data"]
        existing_leaves.append(self._generate_leaf(dataset_file, meta_input))
        self.template.update(data=existing_leaves)

    def _parse_leaves(self, loaded_meta):
        """_parse_leaf loads the leaves from the metadata."""

        existing_leaves = loaded_meta["data"]

        return existing_leaves

    def create(self):

        # create .dataherb folder
        dataherb_folder = self.dataherb_folder
        try:
            os.mkdir(dataherb_folder)
            logger.info("Created ", dataherb_folder)
        except FileExistsError:
            logger.info(
                dataherb_folder, " already exists! Creating metadata.yml file inside."
            )
            pass

        metadata_file = self.metadata_file

        if os.path.isfile(os.path.join(dataherb_folder, metadata_file)):
            logger.error(
                f"File {os.path.join(dataherb_folder, metadata_file)} already exists!"
            )
            raise SystemExit

        with open(os.path.join(dataherb_folder, metadata_file), "w") as fp:
            documents = yaml.dump(self.template, fp)

    def validate(self):
        """validate the existing metadata file"""

        dataherb_folder = self.dataherb_folder
        metadata_file = self.metadata_file
        summary = {}

        try:
            if not os.path.exists(dataherb_folder):
                logger.error(f"Folder {dataherb_folder} does'nt exists!")
                raise Exception(f"Path {dataherb_folder} doesn't exist!")
            if not os.path.isfile(os.path.join(dataherb_folder, metadata_file)):
                logger.error(
                    f"File {os.path.join(dataherb_folder, metadata_file)} doesn'nt exists!"
                )
                raise SystemExit

            with open(os.path.join(dataherb_folder, metadata_file), "r") as fp:
                documents = yaml.load(fp)

            logger.info("loaded metadata ", dataherb_folder)
        except FileExistsError:
            logger.info(
                dataherb_folder, " already exists! Creating metadata.yml file inside."
            )
            pass

        data = self._parse_leaves(documents)

        data_summary = summary.get("data", [])

        leaf_validation = LeafValidation()
        for d in data:
            data_summary.append(leaf_validation.summary(d))

        summary["data"] = data_summary

        return summary


class LeafValidation:
    def __init__(self):
        self.description = "Validate a Leaf in in Herb"

    def summary(self, d):
        summary = {
            "path": self._validate__path(d),
            "format": self._validate__format(d),
            "description": self._validate__description(d),
            "size": self._validate__size(d),
            "fields": self._validate__fields(d),
        }
        return summary

    @staticmethod
    def _validate__path(d):
        """
        _validate__path checks if the file specified in metadata exists and report.

        :param d: one data entry in metadata loaded as dictionary
        :type d: dict
        """
        data_summary_d_path = {}
        data_summary_d_path["value"] = d.get("path")
        if not d.get("path"):
            data_summary_d_path["message"] = MESSAGE_CODE["MISSING"]("path")
            data_summary_d_path["status"] = STATUS_CODE["ERROR"]
        else:
            if not os.path.isfile(d.get("path")):
                data_summary_d_path["message"] = MESSAGE_CODE["FILE_NOT_FOUND"](
                    d.get("path")
                )
                data_summary_d_path["status"] = STATUS_CODE["ERROR"]
            else:
                data_summary_d_path["status"] = STATUS_CODE["SUCCESS"]
                data_summary_d_path["message"] = MESSAGE_CODE["FILE_FOUND"](
                    d.get("path")
                )

        return data_summary_d_path

    @staticmethod
    def _validate__format(d):
        """
        _validate__format checks if the format specified in metadata exists and report.

        :param d: one data entry in metadata loaded as dictionary
        :type d: dict
        """
        key = "format"
        data_summary_d_format = {}
        data_summary_d_format["value"] = d.get(key)
        if not d.get(key):
            data_summary_d_format["message"] = MESSAGE_CODE["MISSING"](key)
            data_summary_d_format["status"] = STATUS_CODE["ERROR"]
        else:
            data_summary_d_format["status"] = STATUS_CODE["SUCCESS"]
            data_summary_d_format["message"] = MESSAGE_CODE["EXISTS"](d.get(key))

        return data_summary_d_format

    @staticmethod
    def _validate__description(d):
        """
        _validate__description checks if the description of file in metadata exists and report.

        :param d: one data entry in metadata loaded as dictionary
        :type d: dict
        """
        key = "description"
        data_summary_d_description = {}
        data_summary_d_description["value"] = d.get(key)
        if not d.get(key):
            data_summary_d_description["message"] = MESSAGE_CODE["MISSING"](key)
            data_summary_d_description["status"] = STATUS_CODE["WARNING"]
        else:
            data_summary_d_description["status"] = STATUS_CODE["SUCCESS"]
            data_summary_d_description["message"] = MESSAGE_CODE["EXISTS"](d.get(key))

        return data_summary_d_description

    @staticmethod
    def _validate__size(d):
        """
        _validate__size checks if the size of file in metadata exists and report.

        :param d: one data entry in metadata loaded as dictionary
        :type d: dict
        """
        key = "size"
        data_summary_d_size = {}
        data_summary_d_size["value"] = d.get(key)
        if not d.get(key):
            data_summary_d_size["message"] = MESSAGE_CODE["MISSING"](key)
            data_summary_d_size["status"] = STATUS_CODE["WARNING"]
        else:
            data_summary_d_size["status"] = STATUS_CODE["SUCCESS"]
            data_summary_d_size["message"] = MESSAGE_CODE["EXISTS"](d.get(key))

        return data_summary_d_size

    @staticmethod
    def _validate__fields(d):
        """
        _validate__fields checks if the fields of file in metadata exists and report.

        :param d: one data entry in metadata loaded as dictionary
        :type d: dict
        """
        key = "fields"
        data_summary_d_fields = {}
        data_summary_d_fields["value"] = f"{len(d.get(key))} fields in total"
        if not d.get(key):
            data_summary_d_fields["message"] = MESSAGE_CODE["MISSING"](key)
            data_summary_d_fields["status"] = STATUS_CODE["WARNING"]
        else:
            name_missing_counter = 0
            description_missing_counter = 0
            for field in d.get(key):
                if not field.get("name"):
                    name_missing_counter += 1
                if not field.get("description"):
                    description_missing_counter += 1
            if (name_missing_counter == 0) and (description_missing_counter == 0):
                data_summary_d_fields["status"] = STATUS_CODE["SUCCESS"]
                data_summary_d_fields["message"] = MESSAGE_CODE["FREE_MESSAGE"](
                    f"{len(d.get(key))} fields: all names and descriptions filled in."
                )
            elif name_missing_counter > 0:
                data_summary_d_fields["status"] = STATUS_CODE["ERROR"]
                data_summary_d_fields["message"] = MESSAGE_CODE["FREE_MESSAGE"](
                    f"{len(d.get(key))} fields: "
                    f"{len(name_missing_counter)} names missing; "
                    f"{len(description_missing_counter)} descriptions missing."
                )
            elif description_missing_counter > 0:
                data_summary_d_fields["status"] = STATUS_CODE["WARNING"]
                data_summary_d_fields["message"] = MESSAGE_CODE["FREE_MESSAGE"](
                    f"{len(d.get(key))} fields: "
                    f"{len(description_missing_counter)} descriptions missing."
                )

        return data_summary_d_fields



import csv
import json
import os
from collections import OrderedDict
from pathlib import Path

import ruamel.yaml
from loguru import logger
from ruamel.yaml.representer import RoundTripRepresenter

IGNORED_FOLDERS_AND_FILES = [".git", ".dataherb", ".vscode"]


MESSAGE_CODE = {
    "MISSING": lambda x: f"{x} is missing",
    "FILE_NOT_FOUND": lambda x: f"{x} was not found",
    "FILE_FOUND": lambda x: f"{x} was found",
    "EXISTS": lambda x: f"{x} exists",
    "FREE_MESSAGE": lambda x: f"{x}",
}

STATUS_CODE = {
    "UNKNOWN": "unknown",
    "SUCCESS": "success",
    "WARNING": "warning",
    "ERROR": "error",
}


class MetaData(object):
    def __init__(self, folder):
        self.dataherb_folder = folder
        self.metadata_file = "dataherb.json"
        self.metadata = {}

    def load(self):
        """load the existing datapackage file"""
        folder = self.dataherb_folder
        metadata_file = Path(folder) / self.metadata_file
        logger.debug(f"Load metadata from file: {metadata_file}")
        with open(metadata_file, "r") as fp:
            self.metadata = json.load(fp)

        logger.debug(f"Loaded metadata: {self.metadata}")

    def create(self):
        """creates dataherb.json file"""
        # create .dataherb folder
        dataherb_folder = self.dataherb_folder
        try:
            os.mkdir(dataherb_folder)
            logger.info("Created ", dataherb_folder)
        except FileExistsError:
            logger.info(
                dataherb_folder,
                " already exists! Creating dataherb.json file inside.",
            )
            pass

        metadata_file = self.metadata_file

        if os.path.isfile(os.path.join(dataherb_folder, metadata_file)):
            logger.error(
                f"File {os.path.join(dataherb_folder, metadata_file)} already exists!"
            )
            raise SystemExit

        with open(os.path.join(dataherb_folder, metadata_file), "w") as fp:
            documents = json.dump(self.metadata, fp, indent=4)

    def validate(self):
        """validate the existing metadata file"""

        dataherb_folder = self.dataherb_folder
        metadata_file = self.metadata_file
        summary = {}

        try:
            if not os.path.exists(dataherb_folder):
                logger.error(f"Folder {dataherb_folder} does'nt exists!")
                raise Exception(f"Path {dataherb_folder} doesn't exist!")
            if not os.path.isfile(os.path.join(dataherb_folder, metadata_file)):
                logger.error(
                    f"File {os.path.join(dataherb_folder, metadata_file)} doesn'nt exists!"
                )
                raise SystemExit

            with open(os.path.join(dataherb_folder, metadata_file), "r") as fp:
                documents = json.load(fp)

            logger.info("loaded metadata ", dataherb_folder)
        except FileExistsError:
            logger.info(
                dataherb_folder, " already exists! Creating metadata.yml file inside."
            )
            pass

        data = self._parse_leaves(documents)

        data_summary = summary.get("data", [])

        summary["data"] = data_summary

        return summary



import os
from pathlib import Path

import click
import git
from datapackage import Package
from loguru import logger
from mkdocs.commands.serve import serve as _serve

from dataherb.cmd.configs import load_dataherb_config
from dataherb.cmd.create import describe_dataset
from dataherb.cmd.sync_git import upload_dataset_to_git
from dataherb.cmd.sync_s3 import upload_dataset_to_s3
from dataherb.core.base import Herb
from dataherb.flora import Flora
from dataherb.parse.model_json import STATUS_CODE, MetaData
from dataherb.serve.save_mkdocs import SaveMkDocs

__CWD__ = os.getcwd()

CONFIG = load_dataherb_config()
logger.debug(CONFIG)
WD = CONFIG["workdir"]
which_flora = CONFIG.get("default", {}).get("flora")
if which_flora:
    which_flora = str(Path(WD) / "flora" / Path(which_flora + ".json"))
    logger.debug(f"Using flora path: {which_flora}")
    if not os.path.exists(which_flora):
        raise Exception(f"flora config {which_flora} does not exist")


@click.group(invoke_without_command=True)
@click.pass_context
def dataherb(ctx):
    if ctx.invoked_subcommand is None:
        click.echo("Hello {}".format(os.environ.get("USER", "")))
        click.echo("Welcome to DataHerb.")
    else:
        click.echo("Loading Service: %s" % ctx.invoked_subcommand)


@dataherb.command()
@click.argument("keywords", required=False)
@click.option("--id", "-i", required=False)
@click.option("--flora", "-f", default=which_flora)
def search(flora, id=None, keywords=None):
    """
    search datasets on DataHerb by keywords or id
    """
    SHOW_KEYS = ["name", "description", "contributors"]
    fl = Flora(flora=flora)
    if not id:
        click.echo("Searching Herbs in DataHerb Flora ...")
        results = fl.search(keywords)
        click.echo(f"Found {len(results)} results")
        if not results:
            click.echo(f"Could not find dataset related to {keywords}")
        else:
            for result in results:
                result_metadata = result.get("herb").metadata
                click.echo(f'DataHerb ID: {result_metadata.get("id")}')
                click.echo(result_metadata)
    else:
        click.echo(f"Fetching Herbs {id} in DataHerb Flora ...")
        result = fl.herb(id)
        if not result:
            click.echo(f"Could not find dataset with id {id}")
        else:
            result_metadata = result.metadata
            click.echo(f'DataHerb ID: {result_metadata.get("id")}')
            click.echo(result_metadata)


@dataherb.command()
@click.option("--flora", "-f", default=which_flora)
@click.option("--workdir", "-w", default=WD, required=True)
@click.option("--dev_addr", "-a", metavar="<IP:PORT>")
def serve(flora, workdir, dev_addr):
    fl = Flora(flora=flora)
    mk = SaveMkDocs(flora=fl, workdir=workdir)
    mk.save_all()

    mkdocs_config = str(Path(WD) / "serve" / "mkdocs.yml")

    click.echo("Open http://localhost:8000")
    _serve(config_file=mkdocs_config, dev_addr=dev_addr)


@dataherb.command()
@click.argument("id", required=True)
@click.option("--flora", "-f", default=which_flora)
def download(id, flora):
    """
    download dataset using id
    """

    fl = Flora(flora=flora)
    click.echo(f"Fetching Herbs {id} in DataHerb Flora ...")
    result = fl.herb(id)
    if not result:
        click.echo(f"Could not find dataset with id {id}")
    else:
        result_metadata = result.metadata
        click.echo(f'Downloading DataHerb ID: {result_metadata.get("id")}')
        result_uri = result_metadata.get("uri")
        result_id = result_metadata.get("id")
        dest_folder = str(Path(WD) / result_id)
        if os.path.exists(dest_folder):
            click.echo(f"Can not download dataset to {dest_folder}: folder exists.\n")

            is_pull = click.confirm(f"Would you like to pull from remote?")
            if is_pull:
                repo = git.Repo(dest_folder)
                repo.git.pull()
            else:
                click.echo(
                    f"Please go to the folder {dest_folder} and sync your repo manually."
                )
        else:
            # dest_folder_parent = f"./{result_repository.split('/')[0]}"
            os.makedirs(dest_folder)
            repo = git.repo.base.Repo.clone_from(result_uri, to_path=dest_folder)
            # git.Git(dest_folder).clone(result_uri)


@dataherb.command()
@click.confirmation_option(
    prompt=f"Your current working directory is {__CWD__}\n"
    "A dataherb.json file will be created right here.\n"
    "Are you sure this is the correct path?"
)
@click.option("--flora", "-f", default=which_flora)
def create(flora):
    """
    creates metadata for current dataset
    """

    use_existing_dpkg = False

    if (Path(__CWD__) / "dataherb.json").exists():
        use_existing_dpkg = click.confirm(
            f"A dataherb.json file already exists in {__CWD__}. "
            f"Shall we use the existing dataherb.json?",
            default=True,
            show_default=True,
        )

    fl = Flora(flora=flora)
    md = MetaData(folder=__CWD__)

    if use_existing_dpkg:
        logger.debug("Using existing dataherb.json ...")
        md.load()
    else:
        dataset_basics = describe_dataset()
        print(dataset_basics)
        md.metadata.update(dataset_basics)

        pkg = Package()
        pkg.infer("**/*.csv")
        pkg_descriptor = {"datapackage": pkg.descriptor}

        md.metadata.update(pkg_descriptor)

        md.create()

        click.echo(
            "The dataherb.json file has been created inside \n"
            f"{__CWD__}\n"
            "Please review the dataherb.json file and update other necessary fields."
        )

    hb = Herb(md.metadata)
    fl.add(hb)

    click.echo(f"Added {hb.metadata['id']} into the flora.")


@dataherb.command()
@click.confirmation_option(
    prompt=f"Your current working directory is {__CWD__}\n"
    "All contents in this folder will be uploaded.\n"
    "Are you sure this is the correct path?"
)
def upload():
    """
    upload dataset in the current folder to the remote destination
    """

    md = MetaData(folder=__CWD__)
    md.load()

    md_uri = md.metadata["uri"]

    is_upload = click.confirm(
        f"The dataset in the current folder\n"
        f"{__CWD__}\n"
        f"will be uploaded to {md_uri}",
        default=True,
        show_default=True,
    )

    if not is_upload:
        click.echo("Upload aborted.")
    else:
        click.echo(f"Uploading dataset to {md_uri} ...")
        if md.metadata.get("source") == "s3":
            upload_dataset_to_s3(__CWD__, md_uri)
        elif md.metadata.get("source") == "git":
            upload_dataset_to_git(__CWD__, md_uri)


@dataherb.command()
@click.option("-v", "--verbose", type=str, default="warning")
def validate(verbose):
    """
    validates the existing metadata for current dataset
    """

    click.secho(
        f"Your current working directory is {__CWD__}\n"
        "I will look for the .dataherb folder right here.\n",
        bold=True,
    )

    ALL_VERBOSE = ["warning", "error", "all"]
    if verbose not in ALL_VERBOSE:
        logger.error(f"-v or --verbose can only take one of {ALL_VERBOSE}")

    md = MetaData()

    validate = md.validate()

    def echo_summary(key, value_dict, bg=None, fg=None):
        if bg is None:
            bg = "black"
        if fg is None:
            fg = "white"
        return click.secho(
            f'  {key}: {value_dict.get("value")}\n'
            f'    STATUS: {value_dict.get("status")};\n'
            f'    MESSAGE: {value_dict.get("message")}',
            bg=bg,
            fg=fg,
        )

    click.secho("Summary: validating metadata:\n- data:", bold=True)
    for val in validate.get("data"):
        for val_key, val_val in val.items():
            if (val_val.get("status") == STATUS_CODE["SUCCESS"]) and (verbose == "all"):
                echo_summary(val_key, val_val, bg="green")
            elif (val_val.get("status") == STATUS_CODE["WARNING"]) and (
                verbose == "warning"
            ):
                echo_summary(val_key, val_val, bg="magenta")
            elif (val_val.get("status") == STATUS_CODE["ERROR"]) and (
                verbose in ["warning", "error"]
            ):
                echo_summary(val_key, val_val, bg="red")

    click.secho(
        "The .dataherb folder and metadata.yml file \n"
        f"{__CWD__}\n"
        " has been validated. Please read the summary and fix the errors.",
        bold=True,
    )


if __name__ == "__main__":
    fl = Flora()
    pass



import json
import logging
import os
import random

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

logging.basicConfig()
logger = logging.getLogger("fetch")


def random_user_agent():
    """Create a list of random headers"""
    user_agent_list = [
        # Chrome
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/60.0.3112.113 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/60.0.3112.90 Safari/537.36",
        "Mozilla/5.0 (Windows NT 5.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/60.0.3112.90 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/60.0.3112.90 Safari/537.36",
        "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/44.0.2403.157 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/60.0.3112.113 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/57.0.2987.133 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/57.0.2987.133 Safari/537.36",
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/55.0.2883.87 Safari/537.36",
        "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/55.0.2883.87 Safari/537.36",
        # Firefox
        "Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1)",
        "Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)",
        "Mozilla/5.0 (Windows NT 6.1; Trident/7.0; rv:11.0) like Gecko",
        "Mozilla/5.0 (Windows NT 6.2; WOW64; Trident/7.0; rv:11.0) like Gecko",
        "Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; rv:11.0) like Gecko",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.0; Trident/5.0)",
        "Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; rv:11.0) like Gecko",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/5.0)",
        "Mozilla/5.0 (Windows NT 6.1; Win64; x64; Trident/7.0; rv:11.0) like Gecko",
        "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; WOW64; Trident/6.0)",
        "Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)",
        "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; "
        ".NET CLR 2.0.50727; .NET CLR 3.0.4506.2152; .NET CLR 3.5.30729)",
    ]

    return {"User-Agent": random.choice(user_agent_list)}


def get_data_from_url(
    link, retry_params=None, headers=None, timeout=None, session=None
):
    """
    get_data_from_url downloads data from the url and return the object

    :param link: link to the data file
    :type link: str
    :param retry_params: retry policies for request session, defaults to None
    :type retry_params: dict, optional
    :param headers: headers for the session, defaults to None
    :type headers: list, optional
    :param timeout: timeout configuration for session, defaults to None
    :type timeout: tuple, optional
    :param session: requests session object, defaults to None
    :type session: requests.sessions.Session, optional
    :return: contens feched from link
    :rtype: requests.models.Response
    """

    if retry_params is None:
        retry_params = {}

    retry_params = {
        **{"retries": 5, "backoff_factor": 0.3, "status_forcelist": (500, 502, 504)},
        **retry_params,
    }

    if headers is None:
        headers = random_user_agent()

    if timeout is None:
        timeout = (5, 14)

    if session is None:
        session = requests.Session()

    retry = Retry(
        total=retry_params.get("retries"),
        read=retry_params.get("retries"),
        connect=retry_params.get("retries"),
        backoff_factor=retry_params.get("backoff_factor"),
        status_forcelist=retry_params.get("status_forcelist"),
    )

    adapter = HTTPAdapter(max_retries=retry)
    session.mount("http://", adapter)
    session.mount("https://", adapter)

    data = session.get(link, headers=headers)

    return data



from dataherb.utils.awscli import aws_cli as _aws_cli


def upload_dataset_to_s3(source, target):
    """
    upload_dataset_to_s3 uploads the dataset to S3
    """

    _aws_cli(("s3", "sync", source, target))



import io
from pathlib import Path

import click
import pandas as pd
from dataherb.cmd.configs import load_dataherb_config
from dataherb.fetch.remote import get_data_from_url as _get_data_from_url
from dataherb.parse.model_json import MetaData
from dataherb.utils.data import flatten_dict as _flatten_dict
from datapackage import Package, Resource
from loguru import logger
from rapidfuzz import fuzz


class Herb(object):
    """
    Herb is a collection of the dataset.
    """

    def __init__(self, meta_dict, base_path=None):
        """
        :param herb_meta_json: the dictionary that specifies the herb
        :type herb_meta_json: dict
        """
        if base_path is None:
            CONFIG = load_dataherb_config()
            self.base_path = CONFIG["workdir"]
        else:
            self.base_path = base_path
        if isinstance(self.base_path, str):
            self.base_path = Path(self.base_path)

        if isinstance(meta_dict, dict):
            self.herb_meta_json = meta_dict
        elif isinstance(meta_dict, MetaData):
            logger.debug("get herb_meta_json from MetaData ...")
            self.herb_meta_json = meta_dict.metadata
        else:
            logger.debug(f"input meta type ({type(meta_dict)}) is not supported.")
            click.BadParameter(
                f"input meta type ({type(meta_dict)}) is not supported.",
                param=meta_dict,
            )

        self._from_meta_dict(self.herb_meta_json)

    @property
    def is_local(self):
        is_local = False
        if self.base_path.exists():
            is_local = True

        return is_local

    def _from_meta_dict(self, meta_dict):
        """Build properties from meta dict"""
        self.name = meta_dict.get("name")
        self.description = meta_dict.get("description")
        self.repository = meta_dict.get("repository")
        self.id = meta_dict.get("id")

        self.source = meta_dict.get("source")
        self.metadata_uri = meta_dict.get("metadata_uri")
        self.datapackage = Package(meta_dict.get("datapackage"))
        if not self.datapackage:
            self.update_datapackage()

        self.resources = [
            self.get_resource(i, source_only=False)
            for i in range(len(self.datapackage.resources))
        ]

    def get_resource(self, idx=None, path=None, name=None, source_only=True):
        if idx is None:
            if path:
                all_paths = [
                    r.descriptor.get("path") for r in self.datapackage.resources
                ]
                if path in all_paths:
                    idx = all_paths.index(path)
                else:
                    logger.error(f"path = {path} is not in resources.")
            elif name:
                all_names = [
                    r.descriptor.get("name") for r in self.datapackage.resources
                ]
                if name in all_names:
                    idx = all_names.index(name)
                else:
                    logger.error(f"name = {name} is not in resources.")
            else:
                raise Exception(
                    f"Please specify at least one of the keywords: idx, path, name."
                )

        if self.is_local:
            logger.info(
                f"Using local dataset for {self.id}, sync it if you need the updated version."
            )
            r = self.datapackage.resources[idx]
            logger.debug(f"using base_path: {str(self.base_path)}")
            logger.debug(f"using descriptor: {r.descriptor}")
            resource = Resource(r.descriptor, base_path=str(self.base_path))
            logger.debug(f"base_path of r_1: {resource._Resource__base_path}")
        elif (not self.is_local) and (self.source == "git"):
            logger.debug(f"Using remote data")
            self.remote_path = f"{self.metadata_uri[:-16]}"
            r = self.datapackage.resources[idx]
            resource = Resource(
                {
                    **(r.descriptor),
                    **{"path": self.remote_path + r.descriptor.get("path", "")},
                }
            )
        elif (not self.is_local) and (self.source == "s3"):
            logger.debug(f"Using remote data")
            logger.debug(
                f"Direct resource from S3 is not supported yet. "
                f"Please sync the dataset to local using the command line first.\n"
                f"TODO: Sync S3 to local after confirmation from here."
            )
            resource = self.datapackage.resources[idx]
        else:
            logger.error("Resource is not supported. Currently supporting S3 and git.")
            resource = self.datapackage.resources[idx]

        if source_only:
            return resource.source
        else:
            return resource

    def update_datapackage(self):
        """
        update_datapackage gets the datapackage metadata from the metadata_uri
        """

        if self.source == "git":
            file_content = _get_data_from_url(self.metadata_uri)

            if not file_content.status_code == 200:
                file_error_msg = "Could not fetch remote file: {}; {}".format(
                    self.url, file_content.status_code
                )
                click.ClickException(file_error_msg)
                # file_content = json.dumps([{"url": self.url, "error": file_error_msg}])
            else:
                file_content = file_content.json()  # .decode(self.decode)
        elif self.source == "s3":
            raise NotImplementedError(
                "Directly get dataherb.json from S3 is not yet implemented."
            )

        self.datapackage_meta = file_content

        self.herb_meta_json["datapackage"] = self.datapackage_meta

        self.datapackage = Package(self.datapackage_meta)

        return self.datapackage

    def search_score(self, keywords, keys=None):
        """
        search_score calcualtes the matching score of the herb for any given keyword

        :param key_word: keyword for the search
        :type key_word: list
        :param keys: list of keys in the dictionary to look into.
        :type keys: list, optional
        """

        if keys is None:
            keys = ["name", "id", "repository", "tags", "description"]

        if not isinstance(keywords, (list, tuple, set)):
            keywords = [keywords]

        herb_for_search = {
            key: val for key, val in self.herb_meta_json.items() if key in keys
        }

        herb_for_search = _flatten_dict(herb_for_search)

        keywords_scores = []
        for keyword in keywords:
            for key, val in herb_for_search.items():
                score = fuzz.token_set_ratio(val, keyword)
                keywords_scores.append(score)

        max_score = 0
        if keywords_scores:
            max_score = max(keywords_scores)

        return max_score

    @property
    def metadata(self, keys=None):
        """
        metadata formats the metadata of the herb
        """

        return self.herb_meta_json.copy()

    def download(self):
        """
        download downloads the dataset
        """

        raise NotImplementedError("Not implemented")

        # data_files = []

        # for leaf_meta in self.herb_meta_json.get("data"):
        #     leaf = Leaf(leaf_meta, self)
        #     data_files.append(leaf.download())

        # return data_files

    def __str__(self):
        meta = self.metadata
        authors = meta.get("contributors", [])
        authors = ", ".join([author.get("name") for author in authors])
        return (
            f"DataHerb ID: {meta.get('id')}\n"
            f"name: {meta.get('name')}"
            f"description: {meta.get('description')}\n"
            f"contributors: {authors}"
        )


class Leaf(object):
    """
    Deprecated

    Leaf is a data file of the Herb.
    """

    def __init__(self, leaf_meta_json, herb):
        self.leaf_meta_json = leaf_meta_json
        self.herb = herb

        self.url = "https://raw.githubusercontent.com/{}/master/{}".format(
            self.herb.repository, self.leaf_meta_json.get("path")
        )
        self.format = self.leaf_meta_json.get("format")
        # decode the file content using decode
        self.decode = self.leaf_meta_json.get("decode", "utf-8")
        self.name = self.leaf_meta_json.get("name")
        self.description = self.leaf_meta_json.get("description")
        self.path = self.leaf_meta_json.get("path")
        self.downloaded = {}

    def download(self):
        """
        download downloads the data
        """

        # Fetch data from remote
        file_content = _get_data_from_url(self.url)
        if not file_content.status_code == 200:
            file_error_msg = "Could not fetch remote file: {}; {}".format(
                self.url, file_content.status_code
            )
            click.ClickException(file_error_msg)
            # file_content = json.dumps([{"url": self.url, "error": file_error_msg}])
        else:
            file_content = file_content.content

        if self.format.lower() == "csv":
            if isinstance(file_content, bytes):
                file_string_io = io.StringIO(file_content.decode(self.decode))
            else:
                file_string_io = file_content
            # csv files may have comment rows
            file_comment = self.leaf_meta_json.get("comment")
            # csv files may have different separators
            file_separator = self.leaf_meta_json.get("seperator", ",")
            try:
                data = pd.read_csv(
                    file_string_io, comment=file_comment, sep=file_separator
                )
            except Exception as e:
                logger.error(f"Error loading remote file: {self.url}")
                data = file_string_io
        elif self.format.lower() == "json":
            if isinstance(file_content, bytes):
                file_string_io = io.StringIO(file_content.decode(self.decode))
            else:
                file_string_io = file_content

            try:
                data = pd.read_json(self.url)
            except Exception as e:
                logger.error(f"Error loading remote file: {self.url}")
                data = file_string_io
        else:
            logger.error(f"data file format {self.format} is not supported!")

        self.downloaded = {"data": data, "content": file_content}

    @property
    def data(self):
        if not self.downloaded:
            self.download()

        return self.downloaded.get("data")

    @property
    def content(self):
        if not self.downloaded:
            self.download()

        return self.downloaded.get("content")

    def metadata(self, format=None):
        """
        metadata formats the metadata of the herb
        """
        if format is None:
            format = "json"

        if format == "json":
            return self.leaf_meta_json
        else:
            logger.error(f"format {format} is not support for metadata!")

    def __str__(self):
        return """{} from {} with size {}, the remote file is located at {};\n\n{}
        """.format(
            self.leaf_meta_json.get("path"),
            self.herb.id,
            self.leaf_meta_json.get("size"),
            self.path,
            self.metadata(),
        )



import json
import os
import click
from pathlib import Path
import time
import yaml

from dataherb.serve.models import SaveModel
from dataherb.serve.mkdocs_templates import site_config as _site_config
from dataherb.serve.mkdocs_templates import index_template as _index_template

from loguru import logger
from slugify import slugify


class SaveMkDocs(SaveModel):
    """
    SaveMkDocs saves the dataset files from source as MkDocs files
    """

    def __init__(self, flora, workdir):

        super().__init__(flora, workdir)

    @staticmethod
    def _generate_markdown_list_meta(dic_lists, name) -> str:
        """
        _markdown_metadata_entry
        """

        if dic_lists:
            md_mkdocs = f"{name}:"
            for l in dic_lists:
                md_mkdocs = md_mkdocs + f'\n  - "{l}"'
            md_mkdocs = md_mkdocs + "\n"
        else:
            md_mkdocs = ""

        return md_mkdocs

    def save_one_markdown(self, herb, path):
        """
        save_one_markdown generates a markdown file
        """

        logger.info(f"Will save {herb.id} to {path}")

        herb_metadata = herb.metadata.copy()
        herb_metadata["title"] = herb_metadata.get("name")
        md_meta = yaml.dump(herb_metadata)

        metadata_mkdocs = f"---\n"
        metadata_mkdocs += md_meta
        metadata_mkdocs += "---\n  "

        with open(path, "w") as fp:
            fp.write(metadata_mkdocs)

        logger.info(f"Saved {herb_metadata} to {path}")

    def save_one_markdown_alt(self, herb, path):
        """
        save_one_markdown generates a markdown file
        """

        logger.info(f"Will save {herb.id} to {path}")

        herb_metadata = herb.metadata

        # generate tilte, description, keywords, and categories
        metadata_title = herb_metadata.get("name")
        metadata_description = herb_metadata.get("description")
        metadata_tags = herb_metadata.get("tags")
        metadata_category = herb_metadata.get("category")

        keywords_mkdocs = self._generate_markdown_list_meta(metadata_tags, "keywords")

        metadata_mkdocs = f'---\ntitle: "{metadata_title}"\n'
        if metadata_description:
            metadata_mkdocs = (
                metadata_mkdocs + f'description: "{metadata_description}"\n'
            )
        if keywords_mkdocs:
            metadata_mkdocs = metadata_mkdocs + keywords_mkdocs
        if metadata_category:
            categories_mkdocs = f"category: {metadata_category}"
            metadata_mkdocs = metadata_mkdocs + categories_mkdocs

        # end the metadata region
        metadata_mkdocs = metadata_mkdocs + "---\n  "

        with open(path, "w") as fp:
            fp.write(metadata_mkdocs)

        logger.info(f"Saved {herb_metadata} to {path}")

    def create_mkdocs_yaml(self):

        mkdocs_folder = Path(self.workdir) / "serve"
        mkdocs_yaml_path = mkdocs_folder / "mkdocs.yml"

        with open(mkdocs_yaml_path, "w") as fp:
            fp.write(_site_config)

    def create_mkdocs_index(self):

        mkdocs_folder = Path(self.workdir) / "serve"
        mkdocs_index_path = mkdocs_folder / "herbs" / "index.md"

        with open(mkdocs_index_path, "w") as fp:
            fp.write(_index_template)

    def save_all(self) -> None:
        """
        save_all saves all files necessary
        """

        # attach working directory to all paths
        md_folder = Path(self.workdir) / "serve" / "herbs"

        # create folders if necessary
        if md_folder.exists():
            is_remove = click.confirm(
                f"{md_folder} exists, remove it and create new?",
                default=True,
                show_default=True,
            )
            if is_remove:
                cache_folder = md_folder.parent / "cache"
                if cache_folder.exists():
                    pass
                else:
                    cache_folder.mkdir(parents=True)
                md_folder.rename(cache_folder / f"serve.{int(time.time())}")
                md_folder.mkdir(parents=True)
        else:
            md_folder.mkdir(parents=True)

        for herb in self.flora.flora:
            herb_id = slugify(herb.id)

            herb_md_path = os.path.join(md_folder, f"{herb_id}.md")
            # generate markdown files
            self.save_one_markdown(herb, herb_md_path)

        self.create_mkdocs_yaml()
        self.create_mkdocs_index()


if __name__ == "__main__":

    pass



import os
import click

try:
    from awscli.clidriver import create_clidriver
except ImportError as e:
    click.echo(f"Please install awscli and config awscli first.")


def aws_cli(*cmd):
    """
    aws_cli invokes the aws cli processes in python to execute awscli commands.
    .. warning::
        This is not the most elegant way of using awscli.
        However, it has been a convinient function in data science projects.
    :param *cmd: tuple of awscli command.
    .. admonition:: Examples
       :class: info
       AWS credential env variables should be configured before calling this function.
       The awscli command should be wrapped as a tuple. To download data from S3 to a local path, use
       >>> aws_cli(('s3', 'sync', 's3://s2-fpd/augmentation/', '/tmp/test'))
       Similarly, upload is done in the following way
       >>> # local_path = ''
       >>> # remote_path = ''
       >>> _aws_cli(('s3', 'sync', local_path, remote_path))
    .. admonition:: References
       :class: info
       This function is adapted from https://github.com/boto/boto3/issues/358#issuecomment-372086466
    """
    old_env = dict(os.environ)
    try:
        # Set up environment
        env = os.environ.copy()
        env["LC_CTYPE"] = "en_US.UTF"
        os.environ.update(env)

        # Run awscli in the same process
        exit_code = create_clidriver().main(*cmd)

        # Deal with problems
        if exit_code > 0:
            raise RuntimeError(f"AWS CLI exited with code {exit_code}")
    finally:
        os.environ.clear()
        os.environ.update(old_env)



site_config = """site_name: My DataHerb

docs_dir: herbs

theme:
  name: "material"
  include_search_page: false
  search_index_only: true

  language: en
  features:
    - navigation.sections
    - navigation.instant
  palette:
    - scheme: default
      primary: indigo
      accent: indigo
      toggle:
        icon: material/toggle-switch-off-outline
        name: Switch to dark mode
    - scheme: slate
      primary: red
      accent: red
      toggle:
        icon: material/toggle-switch
        name: Switch to light mode




markdown_extensions:
  - meta
  - admonition

plugins:
  - search
  - macros
"""


index_template = """---
title: Home
---

This is the landing page of DataHerb.
"""



import os
import click

import inquirer
from dataherb.parse.model_json import IGNORED_FOLDERS_AND_FILES
from loguru import logger


def describe_file(file):
    """
    describe_file [summary]

    [extended_summary]

    :param file: [description]
    :type file: [type]
    :return: [description]
    :rtype: [type]
    """
    questions = [
        inquirer.Text("name", message=f"How would you like to name the file: {file}?"),
        inquirer.Text("description", message=f"What is {file} about?"),
        inquirer.Text(
            "updated_at",
            message=f"When was {file} last updated? In ISO date format such as 2020-02-17.",
        ),
    ]

    answers = inquirer.prompt(questions)
    meta = {
        "name": answers.get("name"),
        "description": answers.get("description"),
        "updated_at": answers.get("updated_at"),
    }

    return meta


def get_metadata_uri(answers):
    """
    get_metadata_uri reconstructs the datapackage uri from the user's answers.
    """

    if answers.get("source") == "git":
        git_repo_link = answers.get("uri")
        git_repo = "/".join(git_repo_link[:-4].split("/")[-2:])
        metadata_uri = (
            f"https://raw.githubusercontent.com/{git_repo}/main/dataherb.json"
        )
    elif answers.get("source") == "s3":
        s3_uri = answers.get("uri", "")
        if s3_uri.endswith("/"):
            s3_uri = s3_uri[:-1]
        metadata_uri = f"{s3_uri}/dataherb.json"
    else:
        click.echo(f'source type {answers.get("source")} is not supported.')

    return metadata_uri


def describe_dataset():
    """
    describe_dataset asks the user to specify some basic info about the dataset
    """
    questions = [
        inquirer.List(
            "source",
            message="Where is/will be the dataset synced to?",
            choices=["git", "s3"],
        ),
        inquirer.Text("name", message="How would you like to name the dataset?"),
        inquirer.Text("id", message="Please specify a unique id for the dataset"),
        inquirer.Text(
            "description",
            message="What is the dataset about? This will be the description of the dataset.",
        ),
        inquirer.Text(
            "uri",
            message="What is the dataset's URI? This will be the URI of the dataset.",
        ),
    ]

    answers = inquirer.prompt(questions)

    metadata_uri = get_metadata_uri(answers)

    meta = {
        "source": answers.get("source"),
        "name": answers.get("name", ""),
        "description": answers.get("description", ""),
        "uri": answers.get("uri", ""),
        "metadata_uri": metadata_uri,
    }

    return meta


def where_is_dataset(directory):
    """
    where_is_dataset asks the user where the dataset is located.
    """
    try:
        folders = []
        for root, dirs, files in os.walk(directory):
            for d in dirs:
                if d not in IGNORED_FOLDERS_AND_FILES:
                    folders.append(os.path.relpath(os.path.join(root, d), "."))
    except Exception as e:
        logger.error("Can not get a list of folders in current directory.")
        folders = []

    folders = [i for i in folders if not i.startswith(".")]

    if folders:
        questions = [
            inquirer.List(
                "dataset_folder",
                message="Which folder contains the data file?",
                choices=folders,
            )
        ]
    else:
        questions = [
            inquirer.Path(
                "dataset_folder",
                message="Which folder will you place the data files?",
                path_type=inquirer.Path.DIRECTORY,
            )
        ]

    answers = inquirer.prompt(questions)
    dataset_folder = answers.get("dataset_folder")

    return dataset_folder



from os import path
from setuptools import setup as _setup
from setuptools import find_packages as _find_packages
from dataherb.version import __version__

# read the contents of your README file
__CWD__ = path.abspath(path.dirname(__file__))
with open(path.join(__CWD__, "README.md"), encoding="utf-8") as fp:
    PACKAGE_LONG_DESCRIPTION = fp.read()


PACKAGE_NAME = "dataherb"
PACKAGE_VERSION = __version__
PACKAGE_DESCRIPTION = "Get clean datasets from DataHerb to boost your data science and data analysis projects"
PACKAGE_URL = "https://github.com/DataHerb/dataherb-python"


def _requirements():
    return [r for r in open("requirements.txt")]


def setup():
    _setup(
        name=PACKAGE_NAME,
        version=PACKAGE_VERSION,
        description=PACKAGE_DESCRIPTION,
        long_description=PACKAGE_LONG_DESCRIPTION,
        long_description_content_type="text/markdown",
        url=PACKAGE_URL,
        author="Lei Ma",
        author_email="hi@leima.is",
        license="MIT",
        packages=_find_packages(exclude=("tests",)),
        install_requires=_requirements(),
        include_package_data=True,
        extras_require={"docs": ["sphinx>=2.4.1", "sphinx-rtd-theme>=0.4.3"]},
        entry_points={"console_scripts": ["dataherb=dataherb.command:dataherb"]},
        test_suite="nose.collector",
        tests_require=["nose"],
        zip_safe=False,
    )


if __name__ == "__main__":
    setup()
    print("Packages: ", _find_packages(exclude=("tests",)))



import click


def upload_dataset_to_git(source, target):

    click.echo(
        f"git sync has not yet been implemented.\n"
        f"Please go to {source} and sync your git repository to {target} manually."
    )



def search_by_keywords_in_flora(flora, keywords, keys=None, min_score=50):
    """
    search_in_flora calculates the match score of each herb and returns the top 10.

    :param flora: list of herbs
    :type flora: list
    :param keywords: search keywords
    :type keywords: list
    :param keys: list of dictionary keys to look into
    :type keys: list, optional
    :param min_score: minimum score of the dataset, default to 50
    :type min_score: float, optional
    :return: herbs that matches the requirements
    :rtype: list
    """

    if not isinstance(keywords, (list, tuple, set)):
        keywords = [keywords]

    herb_scores = []

    for herb in flora:
        herb_search_score = {
            "id": herb.id,
            "herb": herb,
            "score": herb.search_score(keywords),
        }
        herb_scores.append(herb_search_score)

    ranked_herbs = sorted(herb_scores, key=lambda i: i["score"], reverse=True)

    ranked_herbs = [i for i in ranked_herbs if i.get("score") >= min_score]

    return ranked_herbs


def search_by_ids_in_flora(flora, ids):
    """
    search_in_flora finds the herb with the corresponding ids

    :param flora: list of herbs
    :type flora: list
    :param ids: ids of the herbs to be located
    :type ids: list
    :return: herbs that matches the id
    :rtype: list
    """

    if not isinstance(ids, (list, tuple, set)):
        ids = [ids]

    herbs = []
    for herb in flora:
        if herb.id in ids:
            herb_matched = {"herb": herb, "id": herb.id}
            herbs.append(herb_matched)

    return herbs
